import 'dart:async';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:sk_base_mobile/app_theme.dart';
import 'package:sk_base_mobile/models/index.dart';
import 'package:sk_base_mobile/util/debouncer.dart';
import 'package:sk_base_mobile/util/screen_adaper_util.dart';
import 'package:sk_base_mobile/apis/api.dart' as Api;

class ProductSearch extends StatelessWidget {
  ProductSearch({super.key});
  final controller = Get.put(ProductSearchController());
  @override
  Widget build(BuildContext context) {
    return Container(
      padding: EdgeInsets.symmetric(
          horizontal: ScreenAdaper.width(20),
          vertical: ScreenAdaper.height(20)),
      child: Column(
        children: [buildSearchBar(), Expanded(child: buildProductList())],
      ),
    );
  }

  Widget buildSearchBar() {
    final doSearch = debouncer((_) {
      controller.onRefresh();
    }, delayTime: 500);
    return Row(
      children: [
        Expanded(
            child: TextField(
                controller: controller.searchBarController,
                onChanged: (value) => doSearch(value),
                decoration: InputDecoration(
                  contentPadding: EdgeInsets.symmetric(
                      vertical: ScreenAdaper.height(10),
                      horizontal: ScreenAdaper.width(10)),
                  hintText: '名称，编号',
                  floatingLabelBehavior: FloatingLabelBehavior.always,
                  prefixIcon: const Icon(Icons.search),
                  border: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(10),
                  ),
                ))),
      ],
    );
  }

  Widget buildProductList() {
    return Obx(
      () => SmartRefresher(
        enablePullDown: true,
        enablePullUp: true,
        controller: controller.refreshController,
        onLoading: controller.onLoading,
        onRefresh: controller.onRefresh,
        child: ListView.separated(
            itemCount: controller.products.length,
            separatorBuilder: (context, index) => const Divider(
                  height: 1,
                  color: AppTheme.dividerColor,
                ),
            itemBuilder: (context, index) {
              final itemData = controller.products[index];
              return InkWell(
                onTap: () {},
                child: Container(
                  padding:
                      EdgeInsets.symmetric(vertical: ScreenAdaper.height(10)),
                  child: Row(
                    children: [
                      Text(
                        itemData.productNumber!,
                        style: TextStyle(fontSize: ScreenAdaper.sp(20)),
                      ),
                      SizedBox(
                        width: ScreenAdaper.width(10),
                      ),
                      Expanded(
                          child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Text(
                            '${itemData.name}',
                            style: TextStyle(fontSize: ScreenAdaper.sp(20)),
                          ),
                          Text(
                            '${itemData.company?.name}',
                            style: TextStyle(
                                fontSize: ScreenAdaper.sp(15),
                                color: AppTheme.grey),
                          )
                        ],
                      )),
                    ],
                  ),
                ),
              );
            }),
      ),
    );
  }
}

class ProductSearchController extends GetxController {
  RxList<ProductModel> products = RxList([]);
  TextEditingController searchBarController = TextEditingController(text: '');
  RefreshController refreshController = RefreshController(initialRefresh: true);
  int page = 1;
  int limit = 15;
  int total = 0;
  Future<List<ProductModel>> getData({bool isRefresh = false}) async {
    if (isRefresh == true) {
      page = 1;
    } else {
      page++;
    }
    final res = await Api.getProducts(
        {'page': page, 'pageSize': 30, 'keyword': searchBarController.text});
    List<ProductModel> newList =
        res.data!.items.map((e) => ProductModel.fromJson(e)).toList();
    isRefresh == true ? products.assignAll(newList) : products.addAll(newList);

    return newList;
  }

  Future<void> onRefresh() async {
    await getData(isRefresh: true).then((_) {
      refreshController.refreshCompleted(resetFooterState: true);
    }).catchError((_) {
      refreshController.refreshFailed();
    });
  }

  Future<void> onLoading() async {
    await getData().then((_) {
      if (_.isEmpty) {
        refreshController.loadNoData();
      } else {
        refreshController.loadComplete();
      }
    }).catchError((_) {
      refreshController.loadFailed();
    });
  }
}
